@@ -15,7 +15,9 @@ class plymesh : public hitable {
    ~plymesh() {}
   plymesh(std::string inputfile, std::string basedir, std::shared_ptr<material> mat, 
           std::shared_ptr<alpha_texture> alpha, std::shared_ptr<bump_texture> bump,
-          Float scale, Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
+          Float scale, int subdivision_levels, bool recalculate_normals,
+          bool verbose,
+          Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
           std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation);
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler);
