@@ -153,24 +153,29 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
     normal3f n1 = mesh->n[n[0]];
     normal3f n2 = mesh->n[n[1]];
     normal3f n3 = mesh->n[n[2]];
-    normal3f np = unit_vector(b0 * n1 + b1 * n2 + b2 * n3);
-
-    if(mesh->has_consistent_normals) {
-      bool flip = dot(np, r.direction()) > 0;
-      Float af1 = mesh->alpha_v[n[0]];
-      Float af2 = mesh->alpha_v[n[1]];
-      Float af3 = mesh->alpha_v[n[2]];
-      Float af = b0 * af1 + b1 * af2 + b2 * af3;
-      vec3f i = -unit_vector(r.direction());
-      i *= flip ? -1 : 1;
-      Float b = dot(i,np);
-      Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
-      Float g = 1 + q * (b - 1);
-      Float rho = sqrt(q * (1 + g) / (1 +b));
-      normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
-      rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+    
+    normal3f np = (b0 * n1 + b1 * n2 + b2 * n3);
+    if(np.squared_length() == 0) {
+      rec.normal = normal;
     } else {
-      rec.normal = np;
+      np.make_unit_vector();
+      if(mesh->has_consistent_normals) {
+        bool flip = dot(np, r.direction()) > 0;
+        Float af1 = mesh->alpha_v[n[0]];
+        Float af2 = mesh->alpha_v[n[1]];
+        Float af3 = mesh->alpha_v[n[2]];
+        Float af = b0 * af1 + b1 * af2 + b2 * af3;
+        vec3f i = -unit_vector(r.direction());
+        i *= flip ? -1 : 1;
+        Float b = dot(i,np);
+        Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
+        Float g = 1 + q * (b - 1);
+        Float rho = sqrt(q * (1 + g) / (1 +b));
+        normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
+        rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+      } else {
+        rec.normal = np;
+      }
     }
   } else {
     if(alpha_mask) {
@@ -354,24 +359,28 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
     normal3f n1 = mesh->n[n[0]];
     normal3f n2 = mesh->n[n[1]];
     normal3f n3 = mesh->n[n[2]];
-    normal3f np = unit_vector(b0 * n1 + b1 * n2 + b2 * n3);
-    
-    if(mesh->has_consistent_normals) {
-      bool flip = dot(np, r.direction()) > 0;
-      Float af1 = mesh->alpha_v[n[0]];
-      Float af2 = mesh->alpha_v[n[1]];
-      Float af3 = mesh->alpha_v[n[2]];
-      Float af = b0 * af1 + b1 * af2 + b2 * af3;
-      vec3f i = -unit_vector(r.direction());
-      i *= flip ? -1 : 1;
-      Float b = dot(i,np);
-      Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
-      Float g = 1 + q * (b - 1);
-      Float rho = sqrt(q * (1 + g) / (1 +b));
-      normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
-      rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+    normal3f np = (b0 * n1 + b1 * n2 + b2 * n3);
+    if(np.squared_length() == 0) {
+      rec.normal = normal;
     } else {
-      rec.normal = np;
+      np.make_unit_vector();
+      if(mesh->has_consistent_normals) {
+        bool flip = dot(np, r.direction()) > 0;
+        Float af1 = mesh->alpha_v[n[0]];
+        Float af2 = mesh->alpha_v[n[1]];
+        Float af3 = mesh->alpha_v[n[2]];
+        Float af = b0 * af1 + b1 * af2 + b2 * af3;
+        vec3f i = -unit_vector(r.direction());
+        i *= flip ? -1 : 1;
+        Float b = dot(i,np);
+        Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
+        Float g = 1 + q * (b - 1);
+        Float rho = sqrt(q * (1 + g) / (1 +b));
+        normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
+        rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+      } else {
+        rec.normal = np;
+      }
     }
   } else {
     if(alpha_mask) {
@@ -474,7 +483,7 @@ vec3f triangle::random(const point3f& origin, Sampler* sampler, Float time) {
 }
 
 void triangle::GetUVs(point2f uv[3]) const {
-  if (mesh->has_tex && t[0] != -1 && t[1] != -1 && t[1] != -1) {
+  if (mesh->has_tex && t[0] != -1 && t[1] != -1 && t[2] != -1) {
     uv[0] = mesh->uv[t[0]];
     uv[1] = mesh->uv[t[1]];
     uv[2] = mesh->uv[t[2]];
